function KK = IT_Kernel(Test,Train, ITK_type, ITK_param)
% KK = IT_Kernel(Test,Train, ITK_type, ITK_param)
% 
% compute the Information theory kernel
% Test: objects to be characterized with the kernel
% each row is a sample
% 
% Train: objects to which compute the kernel
%
% NOTE: when using  ITK_type=1 or  ITK_type=2, input should
% be normalized before calling this function
% so that sum(Train(i,:)) = 1 for all i
% sum(Test(i,:)) = 1 for all i
%
% ITK_type is the kernel type
% 1 = Shannon-Jensen divergence kernel
% 2 = Jensen Tsallis q-difference kernel (page 947 of Mario JMLR09 paper)
% 3,4 = Weighted Jensen Tsallis kernel (def 26 of Mario JMLR09 paper)
% 3 is the version k_tilde (first of the two)
% the kernel is pd for q = [0,2]
% 4 is se second version: it is pd only for q= [0,1]
%
% ITK_param contains the values of q 
%
% KK is a cell array of kernel matrices 
% (one kernel for each value of the parameter q in ITK_param),
% for each kernel matrix
% each row is the distance between the test object
% and all the training vectors


if ITK_type == 1
    K = zeros(size(Test,1), size(Train,1));
    % Jenson Shannon Divergence
    for j=1:size(Train)
        ytr = Train(j,:);
        h2(j) = entropy(ytr);
    end
    
    for i=1:size(Test,1);
        % kernel train 2 train
        xtr = Test(i,:);
        h1 = entropy(xtr);
        for j=1:size(Train,1);
            ytr = Train(j,:);
            h3 = entropy(0.5*xtr+0.5*ytr);
            JSDiv = h3 - (h1+h2(j))/2;

            K(i,j) = 1 - JSDiv;
        end
        
    end
    KK{1} = K;
end


if ITK_type == 2
    % 2 = Jensen Tsallis q-difference kernel (page 947 of Mario JMLR09 paper)
    ndx = 0;
    for q = ITK_param
        clear h2;
        ndx = ndx +1;
        K = zeros(size(Test,1), size(Train,1));
        % Jenson Shannon Divergence
        for j=1:size(Train,1)
            ytr = Train(j,:);
            h2(j) = TSentropy(ytr,q);
        end

        for i=1:size(Test,1);
            xtr = Test(i,:);
            h1 = TSentropy(xtr,q);
            for j=1:size(Train,1);
                ytr = Train(j,:);
                h3 = TSentropy((0.5*xtr+0.5*ytr),q);
                JTDiv = h3 - (h1+h2(j))/(2^q);

                K(i,j) = 1 - JTDiv;
            end

        end
        KK{ndx} = K;
    end
end


if ITK_type == 3 | ITK_type == 4
    % 3,4 = Weighted Jensen Tsallis kernel (def 26 of Mario JMLR09 paper)
    % 3 is the version k_tilde (first of the two)
    % the kernel is pd for q = [0,2]
    % 4 is se second version: it is pd only for q= [0,1]
    ndx = 0;
    
    % compute all p and w
    for j=1:size(Train,1)
        mu = Train(j,:);
        omega_tr(j) = sum(mu);
        p_tr{j} = mu./omega_tr(j);
    end
    
    for i = 1:size(Test,1);
        mu = Test(i,:);
        omega_te(i) = sum(mu);
        p_te{i} = mu./omega_te(i);
    end 
        
    
    
    for q = ITK_param
        if ITK_type == 4
            if q>1
                q = 1;
            end
        end
        clear h2;
        ndx = ndx +1;
        K = zeros(size(Test,1), size(Train,1));
        for j=1:size(Train,1)
            h2(j) = TSentropy(p_tr{j},q);
        end

        for i=1:size(Test,1);
            h1 = TSentropy(p_te{i},q);
            for j=1:size(Train,1);
                % computation of pi
                den = omega_tr(j) + omega_te(i);
                pi = [omega_tr(j)/den omega_te(i)/den];
            
                h3 = TSentropy((pi(1)*p_tr{j}+pi(2)*p_te{i}),q);
                WJTDiv = h3 - ((pi(1)^q)*h2(j) + (pi(2)^q)*h1);
                
                K(i,j) = (TSentropy(pi,q) - WJTDiv);
                if ITK_type == 3
                    K(i,j) = K(i,j)*(omega_tr(j) + omega_te(i))^q;
                end
            end

        end
        KK{ndx} = K;
    end
end
